@@ -8,7 +8,6 @@ extern crate proc_macro;
 
 use proc_macro::TokenStream;
 use proc_macro2::TokenTree;
-use proc_macro_error::{abort_call_site, proc_macro_error};
 use quote::{format_ident, quote, ToTokens, TokenStreamExt};
 use std::ops::Deref;
 
@@ -62,21 +61,8 @@ use std::ops::Deref;
 /// `test_serial_one` and `test_serial_another` will be executed in serial, as will `test_serial_third` and `test_serial_fourth`
 /// but neither sequence will be blocked by the other
 ///
-/// For each test, a timeout can be specified with the `timeout_ms` parameter to the [serial](macro@serial) attribute. Note that
-/// the timeout is counted from the first invocation of the test, not from the time the previous test was completed. This can
-/// lead to [some unpredictable behavior](https://github.com/palfrey/serial_test/issues/76) based on the number of parallel tests run on the system.
-///
-/// ````
-/// #[test]
-/// #[serial(timeout_ms = 1000)]
-/// fn test_serial_one() {
-///   // Do things
-/// }
-/// ````
-///
 /// Nested serialised tests (i.e. a [serial](macro@serial) tagged test calling another) are supported
 #[proc_macro_attribute]
-#[proc_macro_error]
 pub fn serial(attr: TokenStream, input: TokenStream) -> TokenStream {
     local_serial_core(attr.into(), input.into()).into()
 }
@@ -110,7 +96,6 @@ pub fn serial(attr: TokenStream, input: TokenStream) -> TokenStream {
 /// Note that this has zero effect on [file_serial](macro@file_serial) tests, as that uses a different
 /// serialisation mechanism. For that, you want [file_parallel](macro@file_parallel).
 #[proc_macro_attribute]
-#[proc_macro_error]
 pub fn parallel(attr: TokenStream, input: TokenStream) -> TokenStream {
     local_parallel_core(attr.into(), input.into()).into()
 }
@@ -151,7 +136,6 @@ pub fn parallel(attr: TokenStream, input: TokenStream) -> TokenStream {
 /// ````
 /// Note that in this case you need to specify the `name` arg as well (as per [serial](macro@serial)). The path defaults to a reasonable temp directory for the OS if not specified.
 #[proc_macro_attribute]
-#[proc_macro_error]
 #[cfg_attr(docsrs, doc(cfg(feature = "file_locks")))]
 pub fn file_serial(attr: TokenStream, input: TokenStream) -> TokenStream {
     fs_serial_core(attr.into(), input.into()).into()
@@ -196,7 +180,6 @@ pub fn file_serial(attr: TokenStream, input: TokenStream) -> TokenStream {
 /// ````
 /// Note that in this case you need to specify the `name` arg as well (as per [parallel](macro@parallel)). The path defaults to a reasonable temp directory for the OS if not specified.
 #[proc_macro_attribute]
-#[proc_macro_error]
 #[cfg_attr(docsrs, doc(cfg(feature = "file_locks")))]
 pub fn file_parallel(attr: TokenStream, input: TokenStream) -> TokenStream {
     fs_parallel_core(attr.into(), input.into()).into()
@@ -216,7 +199,6 @@ impl<T: ToTokens> ToTokens for QuoteOption<T> {
 
 enum Arg {
     Name(String),
-    Timeout(u64),
 }
 
 fn get_raw_args(attr: proc_macro2::TokenStream) -> Vec<Arg> {
@@ -226,25 +208,7 @@ fn get_raw_args(attr: proc_macro2::TokenStream) -> Vec<Arg> {
         match attrs.remove(0) {
             TokenTree::Ident(id) => {
                 let name = id.to_string();
-                if name == "timeout_ms" {
-                    match attrs.first() {
-                        Some(TokenTree::Punct(p)) if p.as_char() == '=' && !attrs.is_empty() => {
-                            attrs.remove(0);
-                            if let TokenTree::Literal(lit) = attrs.remove(0) {
-                                let millis = lit
-                                    .to_string()
-                                    .parse::<u64>()
-                                    .expect("Not a valid duration for Timeout");
-                                raw_args.push(Arg::Timeout(millis));
-                            } else {
-                                panic!("Timeout argument must be a literal duration");
-                            }
-                        }
-                        _ => raw_args.push(Arg::Name(name)),
-                    }
-                } else {
-                    raw_args.push(Arg::Name(name));
-                }
+                raw_args.push(Arg::Name(name));
             }
             TokenTree::Literal(literal) => {
                 let string_literal = literal.to_string();
@@ -275,7 +239,6 @@ fn get_raw_args(attr: proc_macro2::TokenStream) -> Vec<Arg> {
 #[derive(Default, Debug)]
 struct Config {
     name: String,
-    timeout: Option<u64>,
     path: Option<String>,
 }
 
@@ -292,9 +255,6 @@ fn get_core_key(attr: proc_macro2::TokenStream) -> Config {
             Arg::Name(name) => {
                 c.path = Some(name);
             }
-            Arg::Timeout(timeout) => {
-                c.timeout = Some(timeout);
-            }
         }
     }
     c
@@ -305,12 +265,7 @@ fn local_serial_core(
     input: proc_macro2::TokenStream,
 ) -> proc_macro2::TokenStream {
     let config = get_core_key(attr);
-    let timeout = if let Some(t) = config.timeout {
-        quote! { ::std::option::Option::Some(::std::time::Duration::from_millis(#t)) }
-    } else {
-        quote! { ::std::option::Option::None }
-    };
-    let args: Vec<Box<dyn ToTokens>> = vec![Box::new(config.name), Box::new(timeout)];
+    let args: Vec<Box<dyn ToTokens>> = vec![Box::new(config.name)];
     serial_setup(input, args, "local")
 }
 
@@ -319,26 +274,14 @@ fn local_parallel_core(
     input: proc_macro2::TokenStream,
 ) -> proc_macro2::TokenStream {
     let config = get_core_key(attr);
-    let timeout = if let Some(t) = config.timeout {
-        quote! { Some(::std::time::Duration::from_millis(#t)) }
-    } else {
-        quote! { None }
-    };
-    let args: Vec<Box<dyn ToTokens>> = vec![Box::new(config.name), Box::new(timeout)];
+    let args: Vec<Box<dyn ToTokens>> = vec![Box::new(config.name)];
     parallel_setup(input, args, "local")
 }
 
 fn fs_args(attr: proc_macro2::TokenStream) -> Vec<Box<dyn ToTokens>> {
     let config = get_core_key(attr);
-    let timeout = if let Some(_t) = config.timeout {
-        panic!("Timeout is not supported for file_serial");
-        // quote! { ::std::option::Option::Some(::std::time::Duration::from_millis(#t)) }
-    } else {
-        quote! { ::std::option::Option::None }
-    };
     vec![
         Box::new(config.name),
-        Box::new(timeout),
         if let Some(path) = config.path {
             Box::new(QuoteOption(Some(path)))
         } else {
@@ -377,45 +320,27 @@ where
     if asyncness.is_some() && cfg!(not(feature = "async")) {
         panic!("async testing attempted with async feature disabled in serial_test!");
     }
+    let vis = ast.vis;
     let name = ast.sig.ident;
     let return_type = match ast.sig.output {
         syn::ReturnType::Default => None,
         syn::ReturnType::Type(_rarrow, ref box_type) => Some(box_type.deref()),
     };
     let block = ast.block;
-    let attrs: Vec<syn::Attribute> = ast
-        .attrs
-        .into_iter()
-        .filter(|at| {
-            if let Ok(m) = at.parse_meta() {
-                let path = m.path();
-                if asyncness.is_some()
-                    && path.segments.len() == 2
-                    && path.segments[1].ident == "test"
-                {
-                    // We assume that any 2-part attribute with the second part as "test" on an async function
-                    // is the "do this test with reactor" wrapper. This is true for actix, tokio and async_std.
-                    abort_call_site!(
-                        "Found async test attribute after serial/parallel, which will break"
-                    );
-                }
-
-                // we skip ignore/should_panic because the test framework already deals with it
-                !(path.is_ident("ignore") || path.is_ident("should_panic"))
-            } else {
-                true
-            }
-        })
-        .collect();
+    let attrs: Vec<syn::Attribute> = ast.attrs.into_iter().collect();
     if let Some(ret) = return_type {
         match asyncness {
             Some(_) => {
                 let fnname = format_ident!("{}_async_{}_core_with_return", prefix, kind);
+                let temp_fn = format_ident!("_{}_internal", name);
                 quote! {
+                    async fn #temp_fn () -> #ret
+                        #block
+
                     #(#attrs)
                     *
-                    async fn #name () -> #ret {
-                        serial_test::#fnname(#(#args ),*, || async #block ).await;
+                    #vis async fn #name () -> #ret {
+                        serial_test::#fnname(#(#args ),*, #temp_fn()).await
                     }
                 }
             }
@@ -424,7 +349,7 @@ where
                 quote! {
                     #(#attrs)
                     *
-                    fn #name () -> #ret {
+                    #vis fn #name () -> #ret {
                         serial_test::#fnname(#(#args ),*, || #block )
                     }
                 }
@@ -434,11 +359,15 @@ where
         match asyncness {
             Some(_) => {
                 let fnname = format_ident!("{}_async_{}_core", prefix, kind);
+                let temp_fn = format_ident!("_{}_internal", name);
                 quote! {
+                    async fn #temp_fn ()
+                        #block
+
                     #(#attrs)
                     *
-                    async fn #name () {
-                        serial_test::#fnname(#(#args ),*, || async #block ).await;
+                    #vis async fn #name () {
+                        serial_test::#fnname(#(#args ),*, #temp_fn()).await;
                     }
                 }
             }
@@ -447,7 +376,7 @@ where
                 quote! {
                     #(#attrs)
                     *
-                    fn #name () {
+                    #vis fn #name () {
                         serial_test::#fnname(#(#args ),*, || #block );
                     }
                 }
@@ -496,63 +425,31 @@ mod tests {
         let compare = quote! {
             #[test]
             fn foo () {
-                serial_test::local_serial_core("",  :: std :: option :: Option :: None, || {} );
+                serial_test::local_serial_core("", || {} );
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
     }
 
     #[test]
-    fn test_serial_with_timeout() {
-        let attrs = vec![
-            TokenTree::Ident(format_ident!("timeout_ms")),
-            TokenTree::Punct(Punct::new('=', Spacing::Alone)),
-            TokenTree::Literal(Literal::u8_unsuffixed(42)),
-        ];
+    fn test_serial_with_pub() {
+        let attrs = proc_macro2::TokenStream::new();
         let input = quote! {
             #[test]
-            fn foo() {}
+            pub fn foo() {}
         };
-        let stream = local_serial_core(
-            proc_macro2::TokenStream::from_iter(attrs.into_iter()),
-            input,
-        );
+        let stream = local_serial_core(attrs.into(), input);
         let compare = quote! {
             #[test]
-            fn foo () {
-                serial_test::local_serial_core("",  :: std :: option :: Option :: Some (::std::time::Duration::from_millis(42u64)), || {} );
+            pub fn foo () {
+                serial_test::local_serial_core("", || {} );
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
     }
 
     #[test]
-    fn test_serial_with_name_and_timeout() {
-        let attrs = vec![
-            TokenTree::Ident(format_ident!("foo")),
-            TokenTree::Punct(Punct::new(',', Spacing::Alone)),
-            TokenTree::Ident(format_ident!("timeout_ms")),
-            TokenTree::Punct(Punct::new('=', Spacing::Alone)),
-            TokenTree::Literal(Literal::u8_unsuffixed(42)),
-        ];
-        let input = quote! {
-            #[test]
-            fn foo() {}
-        };
-        let stream = local_serial_core(
-            proc_macro2::TokenStream::from_iter(attrs.into_iter()),
-            input,
-        );
-        let compare = quote! {
-            #[test]
-            fn foo () {
-                serial_test::local_serial_core("foo",  :: std :: option :: Option :: Some (::std::time::Duration::from_millis(42u64)), || {} );
-            }
-        };
-        assert_eq!(format!("{}", compare), format!("{}", stream));
-    }
-    #[test]
-    fn test_stripped_attributes() {
+    fn test_other_attributes() {
         let _ = env_logger::builder().is_test(true).try_init();
         let attrs = proc_macro2::TokenStream::new();
         let input = quote! {
@@ -565,9 +462,11 @@ mod tests {
         let stream = local_serial_core(attrs.into(), input);
         let compare = quote! {
             #[test]
+            #[ignore]
+            #[should_panic(expected = "Testing panic")]
             #[something_else]
             fn foo () {
-                serial_test::local_serial_core("",  :: std :: option :: Option :: None, || {} );
+                serial_test::local_serial_core("",  || {} );
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
@@ -582,8 +481,9 @@ mod tests {
         };
         let stream = local_serial_core(attrs.into(), input);
         let compare = quote! {
+            async fn _foo_internal () { }
             async fn foo () {
-                serial_test::local_async_serial_core("",  :: std :: option :: Option :: None, || async {} ).await;
+                serial_test::local_async_serial_core("", _foo_internal() ).await;
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
@@ -598,22 +498,14 @@ mod tests {
         };
         let stream = local_serial_core(attrs.into(), input);
         let compare = quote! {
+            async fn _foo_internal ()  -> Result<(), ()> { Ok(()) }
             async fn foo () -> Result<(), ()> {
-                serial_test::local_async_serial_core_with_return("", :: std :: option :: Option :: None, || async { Ok(()) } ).await;
+                serial_test::local_async_serial_core_with_return("", _foo_internal() ).await
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
     }
 
-    // 1.54 needed for https://github.com/rust-lang/rust/commit/9daf546b77dbeab7754a80d7336cd8d00c6746e4 change in note message
-    #[rustversion::since(1.54)]
-    #[test]
-    #[cfg(feature = "async")]
-    fn test_serial_async_before_wrapper() {
-        let t = trybuild::TestCases::new();
-        t.compile_fail("tests/broken/test_serial_async_before_wrapper.rs");
-    }
-
     #[test]
     fn test_file_serial() {
         let attrs = vec![TokenTree::Ident(format_ident!("foo"))];
@@ -628,7 +520,7 @@ mod tests {
         let compare = quote! {
             #[test]
             fn foo () {
-                serial_test::fs_serial_core("foo",  :: std :: option :: Option :: None, None, || {} );
+                serial_test::fs_serial_core("foo", None, || {} );
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
@@ -652,7 +544,7 @@ mod tests {
         let compare = quote! {
             #[test]
             fn foo () {
-                serial_test::fs_serial_core("foo",  :: std :: option :: Option :: None, ::std::option::Option::Some("bar_path"), || {} );
+                serial_test::fs_serial_core("foo", ::std::option::Option::Some("bar_path"), || {} );
             }
         };
         assert_eq!(format!("{}", compare), format!("{}", stream));
